#!/usr/bin/env python3
# -*- encoding: utf-8; py-indent-offset: 4 -*-

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 2 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# If you have not received a copy of the GNU General Public License along
# with this program, see <https://www.gnu.org/licenses/>.


# part of mkp package "powerscale"
# see ~/local/share/doc/check_mk/powerscale for maintainer and information


#<<<powerscale_events>>>
#5611 warning 1680301261.038572 Security Verification Successfully Ran.
#5613 warning 1680301740.068534 Security verification check failed
#5793 critical 1680553852.193812 Battery Backup Unit fault detected

#<ID> <severity> <time> <descr>

#from .agent_based_api.v1 import *
from .agent_based_api.v1 import (
    register,
    render,
    Result,
    State,
    Service,
)

import time


# factory settings:
powerscale_events_factory_settings = {
    "crit_states"      : ['critical', 'emergency'],
    "ignore_list"      : ['Security Verification Successfully Ran.', 
                          'Security verification check failed']
}

# parse function
def parse_powerscale_events(string_table):

    # we'll return this later
    data = {}

    # loop thru
    for line in string_table:
        data[line[0]] = {'severity': line[1], 
                         'time'    : float(line[2]), 
                         'descr'   : ' '.join(line[3:])}

    return data


# discovery function
def discover_powerscale_events(section):
    yield Service()


# check function
def check_powerscale_events(params, section):

    # all fine
    if len(section) == 0:
        yield Result(state=State.OK, summary='No unresolved messages found in event log')
        return 

    # init
    mysummary = []
    mydetails = []
    myignored = []

    # just run thru. any issue found is warn or crit
    for entry in list(section):

        age = float(time.time() - section[entry]['time'])

        # ignored?
        if section[entry]['descr'] in params['ignore_list']:
            myignored.append('[Ignored] %s: "%s" , ID: %s, %s (Age: %s)' % (
                              section[entry]['severity'],
                              section[entry]['descr'], 
                              entry,
                              render.datetime(section[entry]['time']),
                              render.timespan(age)))
            continue

        marker = '(!!)' if section[entry]['severity'] in params['crit_states'] else '(!)'

        mysummary.append('%s: "%s"%s' % (
                            section[entry]['severity'], 
                            section[entry]['descr'],
                            marker))


        mydetails.append('%s: "%s" , ID: %s, %s (Age: %s)' % (
                            section[entry]['severity'],
                            section[entry]['descr'], 
                            entry,
                            render.datetime(section[entry]['time']),
                            render.timespan(age)))

    # ignored? short info to summary, full info to details
    if len(myignored) > 0:
        mysummary.append('%s Entr%s ignored' % (
                          len(myignored),
                          'y' if len(myignored) == 1 else 'ies'))
        mydetails += myignored
        
        
    # output
    mysummary  = ', '.join(mysummary)
    mydetails  = '\n'.join(mydetails)

    # get worst state from markers
    mystate   = State.CRIT if '!!' in mysummary else State.WARN

    # this is the end, my friend
    yield Result(state=mystate, summary=mysummary, details=mydetails)
    return

# register parse function
register.agent_section(
    name                = "powerscale_events",
    parse_function      = parse_powerscale_events,
)

# register check
register.check_plugin(
    name                     = "powerscale_events",
    service_name             = "PowerScale Events",
    discovery_function       = discover_powerscale_events,
    check_function           = check_powerscale_events,
    check_ruleset_name       = "powerscale_events",
    check_default_parameters = powerscale_events_factory_settings,
)
